﻿//     ___                        _                    _     
//    / _ \_   _ _ __  _ __   ___| |_  /\/\   __ _ ___| |_ ___ _ __ 
//   / /_)/ | | | '_ \| '_ \ / _ \ __|/    \ / _` / __| __/ _ \ '__|
//  / ___/| |_| | |_) | |_) |  __/ |_/ /\/\ \ (_| \__ \ ||  __/ |  
//  \/     \__,_| .__/| .__/ \___|\__\/    \/\__,_|___/\__\___|_|  
//              |_|   |_|                    
//                                           
using UnityEngine;

namespace PuppetMaster
{
    public class ChaseCamX : MonoBehaviour
    {
        [SkipSerialisation] public static GameObject ChaseObj;
        [SkipSerialisation] public static PhysicalBehaviour ChasePB;
        [SkipSerialisation] public static PhysicalBehaviour ChaseTarget;

        [SkipSerialisation] public static Puppet Puppet;

        [SkipSerialisation] public static float yOffset;
        [SkipSerialisation] public static float LookAheadX   = 2.0f;
        [SkipSerialisation] public static float ItemTimeout  = 2.0f;
        [SkipSerialisation] public static float yThreshold   = 1.0f;
        [SkipSerialisation] public static float xThreshold   = 2.0f;

        private static bool isMoving           = false;
        private static bool lockedOn           = false;
        private static bool setCamera          = false;
        private static bool checkedMouse       = false;
        private static bool targetClicked      = false;
        private static bool skipFrame          = false;
        private static bool yMoving            = false;

        private static readonly float maxSpeed = 5.0f;

        private static float notMoving         = 0f;
        private static float slerpSpeed        = 5;

        private static ChaseModes ChaseMode    = ChaseModes.Idle;

        private static Vector3 velocity        = Vector3.zero;
        private static Vector3 cTarget;
        private static Vector3 cItem;
        private static Vector3 cPuppet;
        private static Vector3 cItemStill;


        public enum ChaseModes
        {
            Idle,
            Puppet,
            Item,
            Action,
        }


        //
        // ─── UNITY AWAKE ────────────────────────────────────────────────────────────────
        //
        public void Awake()
        {
            CreateChaseObject();
        }


        //
        // ─── UNITY LATE UPDATE ────────────────────────────────────────────────────────────────
        //
        public void LateUpdate()
        {
            if (ChaseTarget == null) return;

            if (skipFrame)
            {
                skipFrame = false;
                return;
            }

            if (setCamera)
            {
                setCamera = false;
                if (!Global.main.CameraControlBehaviour.CurrentlyFollowing.Contains(ChasePB))
                {
                    Global.main.CameraControlBehaviour.CurrentlyFollowing.Clear();
                    Global.main.CameraControlBehaviour.CurrentlyFollowing.Add(ChasePB);
                }

                return;
            }

            //  Prevent chasecam following puppet when its being dragged around the screen
            if (KB.MouseDown)
            {
                if (!checkedMouse)
                {
                    checkedMouse = true;

                    Collider2D[] NoCollide = Puppet.PBO.transform.root.GetComponentsInChildren<Collider2D>();

                    foreach (Collider2D collider in NoCollide)
                    {
                        if (!collider || collider == null) continue;

                        if ((bool)collider?.OverlapPoint((Vector2)Global.main.MousePosition))
                        {
                            targetClicked = true;
                            break;
                        }
                    }
                }

                if (targetClicked) return;
            }
            else checkedMouse = targetClicked = false;

            //  Chase funcs
            if      (ChaseMode == ChaseModes.Puppet) TrackPuppet();
            else if (ChaseMode == ChaseModes.Item)   TrackItem();
        }


        //
        // ─── TRACK PUPPET ────────────────────────────────────────────────────────────────
        //
        private void TrackPuppet()
        {
            if (Puppet == null)
            {
                ChaseMode = ChaseModes.Idle;
                return;
            }

            cTarget     = ChaseObj.transform.position;
            cPuppet     = ChaseTarget.transform.position;

            //  have camera look ahead the direction puppet is facing
            cPuppet.x += LookAheadX * -Puppet.Facing;

            //  have camera Keep the chosen Y offset from the players position
            if (!lockedOn) cPuppet.y += yOffset;

            cPuppet.z = -10f;

            float diffX   = Mathf.Abs(cTarget.x - cPuppet.x);
            float diffY   = Mathf.Abs(cTarget.y - cPuppet.y);
            
            if (isMoving) 
            {  
                if (diffX < 0.2f && diffY < (yMoving ? 0.7f : 1.5f)) {
                
                    if (++notMoving > 15) {
                    
                        notMoving  = 0;
                        lockedOn   = false;
                        isMoving   = false;
                        yMoving    = false;

                    }

                } else notMoving = 0;

            }

            if (isMoving || diffX > xThreshold || diffY > yThreshold)
            {
                float vDistance = Vector3.SqrMagnitude(cTarget - cPuppet);
                float vMaxSpeed = Mathf.Max(maxSpeed, vDistance);

                velocity *= vDistance * Time.deltaTime;

                if (lockedOn) {
                    if (slerpSpeed++ > 25f) slerpSpeed = 25f;
                    ChaseObj.transform.position = Vector3.Slerp(cTarget, cPuppet, slerpSpeed * Time.deltaTime);
                }
                else 
                { 
                    if (vMaxSpeed > 20f && diffY > yThreshold * 3) {
                        slerpSpeed = 5f;
                        lockedOn   = true;
                    }

                    if (diffY < yThreshold) cPuppet.y = cTarget.y;
                    else yMoving = true;

                    isMoving = true;

                    ChaseObj.transform.position = Vector3.SmoothDamp(
                        cTarget,
                        cPuppet,
                        ref velocity, 
                        Time.deltaTime,
                        vMaxSpeed
                    );
                }
            }
        }

        //
        // ─── TRACK ITEM ────────────────────────────────────────────────────────────────
        //
        private void TrackItem()
        {
            if (ChaseTarget == null)
            {
                if (Time.time - notMoving > 2f) StopQuickChase();
                return;
            }

            if (KB.AnyKey)
            {
                StopQuickChase();
                return;
            }

            cTarget = ChaseObj.transform.position;
            cItem   = ChaseTarget.transform.position;

            //float vDistance = Vector3.SqrMagnitude(cTarget - cItem);
            float vDistance = Vector3.SqrMagnitude(cItemStill - cItem);

            if (vDistance > 0.01f) {
                notMoving  = Time.time;
                cItemStill = cTarget;
            }
            else if (Time.time - notMoving > 2f)
            {
                StopQuickChase();
                return;
            }

            ChaseObj.transform.position = cItem;

        }


        //
        // ─── QUICK CHASE ────────────────────────────────────────────────────────────────
        //
        public void QuickChase(PhysicalBehaviour target)
        {
            ChaseTarget = target;
            ChaseMode   = ChaseModes.Item;
            cItemStill  = Vector3.zero;
        }


        public void StopQuickChase()
        {
            if (KB.Modifier) return;
            if (PuppetMaster.isMaxPayne) Util.MaxPayne(false);

            notMoving = 0;
            lockedOn  = false;

            if (Puppet != null)
            {
                SetPuppet(Puppet);
                return;
            }

            ChaseMode = ChaseModes.Idle;
        }


        //
        // ─── SET PUPPET ────────────────────────────────────────────────────────────────
        //
        public void SetPuppet(Puppet puppet, bool resetY=false)
        {
            if (puppet == null) return;
            
            Puppet = puppet;

            if (ChaseObj == null) CreateChaseObject();

            ChaseTarget = Puppet.LB?["Head"].PhysicalBehaviour;

            //  Set the constant Yoffset
            if (resetY) {
                
                yOffset = Global.main.CameraControlBehaviour.transform.position.y - ChaseTarget.transform.position.y;

            }

            //  Start invisible tracker at puppet position
            Vector3 cPuppet = ChaseTarget.transform.position;

            //  have camera look ahead the direction puppet is facing
            cPuppet.x += LookAheadX * -Puppet.Facing;

            //  have camera Keep the chosen Y offset from the players position
            cPuppet.y += yOffset;
            
            ChaseObj.transform.position = cPuppet;

            ChaseMode = ChaseModes.Puppet;
            setCamera = true;
            lockedOn  = false;
            skipFrame = true;
        }


        //
        // ─── CREATE CHASE OBJECT ────────────────────────────────────────────────────────────────
        //
        public void CreateChaseObject()
        {
            //  Create a hidden chase-cam object that the camera follows
            Texture2D tex   = new Texture2D(0, 0);
            Sprite mySprite = Sprite.Create(tex, new Rect(0.0f, 0.0f, tex.width, tex.height), new Vector2(0.1f, 0.1f), 0.1f);
            ChaseObj        = ModAPI.CreatePhysicalObject("ChaseCam", mySprite);

            // Initialize chase-cam object
            ChaseObj.transform.position = Global.CameraPosition;

            ChasePB = ChaseObj.GetOrAddComponent<PhysicalBehaviour>();

            ChasePB.MakeWeightless();
            ChasePB.Selectable = false;
            ChasePB.Deletable  = false;

            //  Prevent all collisions with invisible chase object
            ChaseObj.SetLayer(10);

            //  Prevent this from feking up anyones contraption
            ChaseObj.GetOrAddComponent<Optout>();

            PuppetMaster.ChaseCam = this;

            DontDestroyOnLoad(ChaseObj);
        }

        //
        // ─── STOP (self-destruct) ────────────────────────────────────────────────────────────────
        //
        public void Stop()
        {
            if (Global.main.CameraControlBehaviour.CurrentlyFollowing.Contains(ChasePB))
            {
                Global.main.CameraControlBehaviour.CurrentlyFollowing.Remove(ChasePB);
            }

            if (ChasePB  != null) UnityEngine.Object.Destroy(ChasePB);    
            if (ChaseObj != null) UnityEngine.Object.Destroy(ChaseObj);    

            this.enabled = false;
            UnityEngine.Object.Destroy(this);
        }
    }
}